@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2018	PgPool Global Development Group
+ * Copyright (c) 2003-2020	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -109,6 +109,7 @@
 #include "utils/sha2.h"
 #include "auth/pool_passwd.h"
 #include "auth/scram.h"
+#include "auth/pool_auth.h"
 #include "utils/base64.h"
 #include "utils/elog.h"
 #include "utils/palloc.h"
@@ -502,6 +503,9 @@ scram_verify_plain_password(const char *username, const char *password,
 	{
 		ereport(LOG,
 				(errmsg("invalid SCRAM verifier for user \"%s\"", username)));
+
+		pfree(encoded_salt);
+		pfree(salt);
 		return false;
 	}
 
@@ -514,6 +518,7 @@ scram_verify_plain_password(const char *username, const char *password,
 	 * user-supplied password.
 	 */
 	pfree(encoded_salt);
+	pfree(salt);
 	return memcmp(computed_key, server_key, SCRAM_KEY_LEN) == 0;
 }
 
@@ -592,6 +597,8 @@ parse_scram_verifier(const char *verifier, int *iterations, char **salt,
 	if (decoded_len != SCRAM_KEY_LEN)
 		goto invalid_verifier;
 
+	pfree(v);
+
 	return true;
 
 invalid_verifier:
