@@ -137,6 +137,11 @@ struct timeval startTime;
 bool		stop_now = false;
 #endif
 
+/*
+ * If true, connection pool has been initialized.
+ */
+static bool connection_pool_initialized = false;
+
 /*
 * child main loop
 */
@@ -223,6 +228,7 @@ do_child(int *fds)
 	{
 		child_exit(POOL_EXIT_AND_RESTART);
 	}
+	connection_pool_initialized = true;
 
 	/*
 	 * Open pool_passwd in child process.  This is necessary to avoid the file
@@ -1296,7 +1302,7 @@ child_will_go_down(int code, Datum arg)
 	}
 
 	/* let backend know now we are exiting */
-	if (pool_connection_pool)
+	if (connection_pool_initialized)
 		close_all_backend_connections();
 }
 void
@@ -1730,9 +1736,10 @@ check_config_reload(void)
 		if (pool_config->enable_pool_hba)
 		{
 			load_hba(get_hba_file_name());
-			if (strcmp("", pool_config->pool_passwd))
-				pool_reopen_passwd_file();
 		}
+		if (strcmp("", pool_config->pool_passwd))
+			pool_reopen_passwd_file();
+
 		got_sighup = 0;
 	}
 }
