@@ -145,9 +145,7 @@ pool_signal(int signo, pool_sighandler_t func)
 
 	act.sa_handler = func;
 	sigemptyset(&act.sa_mask);
-	act.sa_flags = 0;
-	if (signo != SIGALRM)
-		act.sa_flags |= SA_RESTART;
+	act.sa_flags = SA_RESTART;
 #ifdef SA_NOCLDSTOP
 	if (signo == SIGCHLD)
 		act.sa_flags |= SA_NOCLDSTOP;
